from escape.core.average import *
from escape.core.distribution import *
from escape.core.objects import *
from escape.core.setting import *
from escape.scattering.diffreftrans import *
from escape.scattering.ff import *
from escape.scattering.geometry import *
from escape.scattering.lattice import *
from escape.scattering.layer import *
from escape.scattering.mdb import *
from escape.scattering.polreftrans import *
from escape.scattering.potential import *
from escape.scattering.profile import *

# from escape.scattering.sas import *
from escape.scattering.reftrans import *
from escape.scattering.spechrxrd import *
from escape.scattering.specrefl import *
from escape.serialization import *
from escape.utils.widgets import show
from escape.utils.widgets.base import LayoutConfig
from escape.utils.widgets.data import DataConfig
from escape.utils.widgets.kernel import KernelConfig
from escape.utils.widgets.model import ModelConfig
from escape.version import __version__

from escape.core.array import *
from escape.core.data import *
from escape.core.kernel import *
from escape.core.math import *
from escape.core.model import *
from escape.core.optimizer import *
from escape.core.peak import *
from escape.core.regressor import *
from escape.scattering.material import *
from escape.scattering.strfac import *

angstr = "\u00c5"


def require(version):
    """Require can be used to check the minimum version required to run a ESCAPE
    application. For example, you can start your application code like this::

        import escape
        escape.require('1.0.1')

    If a user attempts to run your application with a version of ESCAPE that is
    older than the specified version, an Exception is raised.

    The ESCAPE version string is built like this::

        X.Y.Z[-tag[-tagrevision]]

        X is the major version
        Y is the minor version
        Z is the bugfixes revision

    The tag is optional, but may be one of 'dev', 'alpha', or 'beta'.
    The tagrevision is the revision of the tag.

    .. warning::

        You must not ask for a version with a tag, except -dev. Asking for a
        'dev' version will just warn the user if the current ESCAPE
        version is not a -dev, but it will never raise an exception.
        You must not ask for a version with a tagrevision.

    """

    def parse_version(version):
        # check for tag
        tag = None
        tagrev = None
        if "-" in version:
            v = version.split("-")
            if len(v) == 2:
                version, tag = v
            elif len(v) == 3:
                version, tag, tagrev = v
            else:
                raise Exception("Revision format must be X.Y.Z[-tag]")

        # check x y z
        v = version.split(".")
        if len(v) != 3:
            if "dev0" in v:
                tag = v.pop()
            else:
                raise Exception("Revision format must be X.Y.Z[-tag]")
        return [int(x) for x in v], tag, tagrev

    # user version
    revision, tag, tagrev = parse_version(version)
    # current version
    sysrevision, systag, _ = parse_version(__version__)

    # ensure that the required version don't contain tag, except dev
    if tag not in (None, "dev"):
        raise Exception('Revision format must not have any tag except "dev"')
    if tag == "dev" and systag != "dev":
        print(
            "Application requested a -dev version of ESCAPE. "
            "(You have %s, but the application requires %s)" % (__version__, version)
        )
    # not tag rev (-alpha-1, -beta-x) allowed.
    if tagrev is not None:
        raise Exception("Revision format must not contain any tagrevision")

    # finally, checking revision
    if sysrevision < revision:
        raise Exception(
            "The version of ESCAPE installed on this system "
            "is too old. "
            "(You have %s, but the application requires %s)" % (__version__, version)
        )

